# Compute and memory estimations for training LLM's

There are a few scripts here useful for predicting compute/memory requirements
for training large language models. They are quite basic, assuming no particular
underlying hardware or software framework, however they give a good baseline,
which can then be iterated upon.

The estimations are given based on the usual hyperparameters chosen when
training a Transformer-type LLM: number of layers, number of heads, embeddings
dimension, batch size, training context length, vocabulary size. These should
give a rough idea as to the memory requirements for a classical GPT-style
decoder-only Transformer. Other modifications on top of the architecture (such
as SwiGLU activations inside the feedforward layer, multi-query or grouped-query
attention, rotary positional embeddings etc.) will likely change the values by a
little bit, however not enough to justify a totally different approach to the
estimations.

Other types of model architectures (such as mixture of experts, BERT-style
models etc.) should not rely too much on the estimations given by the scripts,
since those use fundamentally different approaches.

The estimations, as mentioned above, do not take into consideration any
particular underlying hardware. What this means is *the estimations assume a
perfect world with a single GPU, with infinite memory and infinite compute*.
Network communication, model-splitting, parallelism and other such things are
not considered in the script. Most likely however, you can assume a 20-40%
overhead on memory, and possibly a little bit more on compute (usual GPU
utilization with hundreds of GPU's is 30-40% per GPU, meaning that 60-70% of the
amount of GPU-hours that you have will most likely be wasted on communication).

The scripts also do not consider techniques such as mixed precision training or
gradient checkpointing which will almost certainly result in less memory and
compute used. These, however, depend very much on the particular setup and what
the user expects from the memory / compute trade-off. As a rule of thumb, mixed
precision will decrease training time by 20-40%, and gradient checkpointing can
reduce memory usage by quite a large amount, trading memory used for compute
(meaning more training time needed overall). Therefore, the estimations should
be taken as a worst-case scenario in this regard.

## Memory requirements

Memory requirements are given by the script `memory_req.py`. Change the values
at the top (or use the predefined defaults), run it and get the output.
